### Custom Keyboard Maestro Plug-in
#### NAME
- Read (text and HTML attributes of) XPath matches in Chrome or Safari

#### VERSION
- 0.1

#### SYNOPSIS
- Returns a JSON list of HTML element properties for the first node (or all nodes) matched by the supplied [XPath](https://developer.mozilla.org/en-US/docs/Web/XPath)
- The XPath can be applied to:
	- the root of the whole **document**,
	- the start of the current **selection**,
	- or the current hover position of the **mouse**.
- In addition to the properties of each match, the JSON returned also includes:
	- The URL and title of the document, and
	- the element properties of any anchor node (selected, or under the mouse cursor) to which the XPath has been applied. ( See the `applied to` option below )
- The text of the matches can be returned as Markdown, raw HTML, or plain unmarked text.
- NB In Yosemite, Keyboard Maestro's `Execute JavaScript for Applications` return a default 'may not be scriptable' error message from osascript when a compiled .scpt file is called. 
	- **This action will unfortunately fail (generating an unparseable string instead of JSON) unless you switch off "Include Errors" in the upper right cogwheel dropdown of the action**


#### OPTIONS

- Read
	- all matches
	- first match only
	
- Xpath:
	- An [XPath expression](https://developer.mozilla.org/en-US/docs/Web/XPath)
			
		**Note 1:** an XPath applied to a mouse position or selection should start with a dot `./`, referring to the selected or hovered node

		**Note 2:** In Google Chrome, a simple absolute XPath for a selected element can be obtained with:
			- Right-Click, **Inspect Element**, then
			- Right-Click, **Copy XPath**

		(The copied XPath will work in Safari, as well as in Chrome itself – Safari does not have its own *Copy XPath* feature)
			
- Applied to:
	- `document`
	- `mouse position`
	- `selection`

- Text as:
	- Markdown
	- [inner HTML](https://developer.mozilla.org/en-US/docs/Web/API/Element/innerHTML)
	- [outer HTML](https://developer.mozilla.org/en-US/docs/Web/API/Element/outerHTML)
	- plain [text content](https://developer.mozilla.org/en-US/docs/Web/API/Node/textContent)
	
-  Browser:
	- `Chrome or Safari`
	- `Safari`
	- `Google Chrome`

#### REQUIREMENTS
- Yosemite
	- The core script `readXPathMatches.scpt` is written in Javascript for Applications

#### INSTALLATION
- Drag the .zip file onto the Keyboard Maestro icon in the OS X toolbar.
- (if updating a previous version of the action, first manually remove the previous copy from the custom actions folder)
	- `~/Library/Application Support/Keyboard Maestro/Keyboard Maestro Actions`
- - NB In Yosemite, Keyboard Maestro's `Execute JavaScript for Applications` return a default 'may not be scriptable' error message from osascript when a compiled .scpt file is called. 
	- **This action will unfortunately fail (generating an unparseable string instead of JSON) unless you switch off "Include Errors" in the upper right cogwheel dropdown of the action**

#### CONTACT
- Rob Trew - Twitter @ComplexPoint